@@ -6,7 +6,7 @@ from aiohttp import ClientError
 from aiohttp.http_exceptions import HttpProcessingError
 import async_timeout
 import jsonrpc_base
-from jsonrpc_base import JSONRPCError, TransportError, ProtocolError
+from jsonrpc_base import TransportError
 
 
 class Server(jsonrpc_base.Server):
@@ -14,15 +14,22 @@ class Server(jsonrpc_base.Server):
 
     def __init__(self, url, session=None, **connect_kwargs):
         super().__init__()
-        object.__setattr__(self, 'session', session or aiohttp.ClientSession())
+        self._session = session or aiohttp.ClientSession()
+
+        # True if we made our own session
+        self._internal_session = session is None
+
         self._client = None
         self._connect_kwargs = connect_kwargs
         self._url = url
-        self._connect_kwargs['headers'] = self._connect_kwargs.get('headers', {})
-        self._connect_kwargs['headers']['Content-Type'] = self._connect_kwargs['headers'].get(
-            'Content-Type', 'application/json')
-        self._connect_kwargs['headers']['Accept'] = self._connect_kwargs['headers'].get(
-            'Accept', 'application/json-rpc')
+        self._connect_kwargs['headers'] = self._connect_kwargs.get(
+            'headers', {})
+        self._connect_kwargs['headers']['Content-Type'] = (
+            self._connect_kwargs['headers'].get(
+                'Content-Type', 'application/json'))
+        self._connect_kwargs['headers']['Accept'] = (
+            self._connect_kwargs['headers'].get(
+                'Accept', 'application/json-rpc'))
         self._timeout = self._connect_kwargs.get('timeout')
         self._pending_messages = {}
 
@@ -53,11 +60,11 @@ class Server(jsonrpc_base.Server):
             raise TransportError('Connection already open.')
 
         try:
-            self._client = await self.session.ws_connect(
+            self._client = await self._session.ws_connect(
                 self._url, **self._connect_kwargs)
         except (ClientError, HttpProcessingError, asyncio.TimeoutError) as exc:
             raise TransportError('Error connecting to server', None, exc)
-        return self.session.loop.create_task(self._ws_loop())
+        return self._session.loop.create_task(self._ws_loop())
 
     async def _ws_loop(self):
         """Listen for messages from the websocket server."""
@@ -86,8 +93,7 @@ class Server(jsonrpc_base.Server):
                     # cpython's optimizations prevent coveragepy from detecting
                     # that it's run
                     # https://bitbucket.org/ned/coveragepy/issues/198/continue-marked-as-not-covered
-                    continue # pragma: no cover
-
+                    continue  # pragma: no cover
 
                 if 'method' in data:
                     request = jsonrpc_base.Request.parse(data)
@@ -102,13 +108,20 @@ class Server(jsonrpc_base.Server):
         finally:
             await self.close()
             if msg and msg.type == aiohttp.WSMsgType.ERROR:
-                raise TransportError('Websocket error detected. Connection closed.')
+                raise TransportError(
+                    'Websocket error detected. Connection closed.')
 
     async def close(self):
         """Close the connection to the websocket server."""
         if self.connected:
             await self._client.close()
             self._client = None
+        if self._internal_session:
+            # If we created a clientsession for this Server, close it here.
+            # And then instantiate a new clientsession in case the
+            # connection should be reopened
+            await self._session.close()
+            self._session = aiohttp.ClientSession()
 
     @property
     def connected(self):
